"""Module that updates the domain model of :term:`GSO`. Should contain all types of subscriptions.

.. warning::
   Whenever a new product is added, this should be reflected in the :py:class:`gso.products.ProductType` enumerator.
   This does not hold for adding a new type of already existing product.
"""

from orchestrator.domain import SUBSCRIPTION_MODEL_REGISTRY
from pydantic_forms.types import strEnum

from gso.products.product_types.ias_gateway import IASGateway
from gso.products.product_types.iptrunk import Iptrunk
from gso.products.product_types.router import Router
from gso.products.product_types.site import Site


class ProductType(strEnum):
    """An enumerator of available products in :term:`GSO`."""

    SITE = "Site"
    NOKIA_ROUTER = "Nokia router"
    JUNIPER_ROUTER = "Juniper router"
    IP_TRUNK = "IP trunk"
    IAS_GATEWAY = "IAS gateway"
    ROUTER = "Router"


SUBSCRIPTION_MODEL_REGISTRY.update(
    {
        "IAS gateway": IASGateway,
        "IP trunk": Iptrunk,
        "Juniper router": Router,
        "Nokia router": Router,
        "Site": Site,
    },
)
